//
//
//
#include <tap.h>

//Every TAP application needs unique ID and TAP allocations that have same ID cant execute at same time.
//ID consists of DWORD( 4byte ) and the MSB bit should be set by 1.
//The following is how to use in source codes.
#define ID_USERTAP 0xbdb00b01
TAP_ID( ID_USERTAP );

//Every TAP application has to include Program name, Author name, and Program description as following
//manners in source code.
TAP_PROGRAM_NAME("epg_stick v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("prevents channel change in EPG");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//

int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int exTAP_EPG_Stick(int enable);

#define pSysID 0xa3fffffa
extern int _appl_version;
char *tap_name;
dword fw_tap_table = 0;

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
typedef struct
{
    byte signature[32];					//TFAMIPS
    dword id;
    byte padding1[28];
    const char name[MAX_PROGRAM_NAME];
    const char authorName[MAX_AUTHOR_NAME];
    const char description[MAX_DESCRIPTION];
    const char etcStr[MAX_ETCINFO_STR];
} type_TAPHeader;

typedef struct
{
    // From libtap.a
    dword _initData;
    void *TAP_Main;
    dword _tap_gotStart;
    dword _tap_gotEnd;
    dword _tap_textEnd;
    dword _tap_fixupStart;
    dword _tap_fixupEnd;
    void *TAP_EventHandler;
    dword _gp;
    dword _end;

    void *pTAP_SystemProc;
    void *_appl_version;
    void *spare0;
    void *pTAP_GetState;
    void *pTAP_ExitNormal;
    //etc
} type_TAPEntryPoints;

typedef struct
{
    dword firmwaregp;
    dword tapgp;
    dword unknown1;                         //tap malloc base address, before + 7ffe, & 0xFFFF8000
    type_TAPHeader *header;                 //after & 0xffff8000
    type_TAPEntryPoints *entryPoints;
    dword loadAddress;
    dword unknown2;                         //this taps current directory ?
    dword zero;
} type_TAPProcess;

bool TSRCommanderExitTAP()
{
    exTAP_EPG_Stick(0);
    TSRCommanderErase();
    TAP_Print("\nClosing %s", tap_name);
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
}

int is_tsr_commander_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
int TAP_Main(void)
{
    word SysID = *(word*)pSysID;

    tap_name =  (char *)__tap_program_name__;
    TAP_Print("\nOpening %s", tap_name);

    //determine support for different firmwares
    switch(SysID) {
        case 456:
            break;
        default:
            TAP_Print("\nUnsupported system id - %d", SysID);
            TAP_Print("\n");
            return -1;
    }
    switch(_appl_version) {
        case 0x1225:
            fw_tap_table = 0x8017187c;
            break;
        default:
            TAP_Print("\nUnsupported firmware version - %04x", _appl_version);
            TAP_Print("\n");
            return -1;
    }

    TSRCommanderInit(0, FALSE);

    //patch the epg to prevent channel change
    exTAP_EPG_Stick(1);

    return 1;                 //0=application mode, 1=tsr mode
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
    switch (event) {
        case EVT_IDLE:
            TSRCommanderWork();
            break;
    }
    return param1;
}

//----------------------------------------------------------------------------
//
//eit stick patch
//
//this patch changes the epg to prevent channel change when viewing other epg channels
//
//

int exTAP_EPG_Stick(int enable)
{
    if (enable) {
        TAP_Print("\nApplying EPG Stick Patch");
        if (*(dword *)0x80120778 == 0x0018c7c2) *(dword *)0x80120778 = 0x24180001;
    }
    else {
        TAP_Print("\nRemoving EPG Stick Patch");
        if (*(dword *)0x80120778 == 0x24180001) *(dword *)0x80120778 = 0x0018c7c2;
    }
}


//----------------------------------------------------------------------------
